@@ -8,6 +8,7 @@ from typing_extensions import Literal, NotRequired, Protocol, TypedDict, TypeVar
 from .hybrid_observer import HybridObservable
 
 # Literals
+FrameType = Literal["video", "motion"]
 CameraType = Literal["camera", "doorbell"]
 ZoneType = Literal["intersect", "contain"]
 ZoneFilter = Literal["include", "exclude"]
@@ -98,16 +99,11 @@ CameraRoles = Literal["high-resolution", "mid-resolution", "low-resolution", "sn
 DecoderFormat = Literal["yuv", "rgb"]
 ImageInputFormat = Literal["yuv", "rgb", "rgba", "gray"]
 ImageOutputFormat = Literal["rgb", "rgba", "gray"]
-CameraExtension = Literal["hub", "motionDetection", "objectDetection", "audioDetection", "ptz"]
+CameraExtension = Literal[
+    "cameraController", "hub", "motionDetection", "objectDetection", "audioDetection", "ptz"
+]
 CameraFrameWorkerDecoder = Literal["pillow", "wasm", "rust"]
 CameraFrameWorkerResolution = Literal[
-    "3840x2160",
-    "3072x1728",
-    "2560x1440",
-    "1920x1080",
-    "1440x1080",
-    "1280x720",
-    "960x540",
     "640x480",
     "640x360",
     "320x240",
@@ -124,26 +120,27 @@ InternalSourceType = Literal["aac", "opus", "pcma"]
 StateNames = Literal["light", "motion", "audio", "doorbell", "siren", "battery", "object"]
 CameraPublicProperties = Literal[
     "_id",
-    "nativeId",
-    "pluginId",
-    "name",
+    "audioDetectionPluginId",
+    "cameraControllerPluginId",
+    "detectionSettings",
+    "detectionZones",
     "disabled",
-    "isCloud",
+    "frameWorkerSettings",
+    "hasBattery",
+    "hasBinarySensor",
     "hasLight",
     "hasSiren",
-    "hasBinarySensor",
-    "hasBattery",
     "info",
-    "type",
-    "detectionZones",
-    "detectionSettings",
-    "hasAudioDetector",
-    "hasMotionDetector",
-    "hasObjectDetector",
-    "extendedMotionDetector",
-    "hasPtz",
+    "isCloud",
+    "motionDetectionPluginId",
+    "name",
+    "nativeId",
+    "objectDetectionPluginId",
+    "pluginId",
+    "ptzPluginId",
+    "snapshotTTL",
     "sources",
-    "frameWorkerSettings",
+    "type",
 ]
 DeviceManagerEventType = Literal[
     "cameraSelected",
@@ -237,17 +234,11 @@ class CameraFrameWorkerSettings(TypedDict):
     resolution: CameraFrameWorkerResolution
 
 
-class StreamUrls(TypedDict):
-    ws: "Go2RtcWSSource"
-    rtsp: "Go2RtcRTSPSource"
-    www: "Go2RtcEndpoint"
-
-
 class CameraInput(TypedDict):
     _id: str
     name: str
     roles: list[CameraRoles]
-    urls: StreamUrls
+    urls: "StreamUrls"
     internal: bool
 
 
@@ -302,6 +293,12 @@ class ProbeStream(TypedDict):
     video: list[VideoStreamInfo]
 
 
+class StreamUrls(TypedDict):
+    ws: "Go2RtcWSSource"
+    rtsp: "Go2RtcRTSPSource"
+    www: "Go2RtcEndpoint"
+
+
 class Go2RtcRTSPSource(TypedDict):
     default: str
     defaultMicrophone: str
@@ -337,6 +334,7 @@ class Go2RtcSource(TypedDict):
 
 class Go2RtcWSSource(TypedDict):
     webrtc: str
+    mse: str
 
 
 T = TypeVar(
@@ -472,8 +470,10 @@ class SetValues(TypedDict):
 
 
 class FrameData(TypedDict):
-    frameId: str
+    data: bytes
     timestamp: int
+    metadata: "FrameMetadata"
+    info: "ImageInformation"
 
 
 class FrameMetadata(TypedDict):
@@ -524,12 +524,21 @@ class FrameBuffer(TypedDict):
     info: ImageInformation
 
 
+class MotionDetectionFrame(TypedDict):
+    frame: FrameData
+    state: MotionSetEvent
+
+
 class VideoFrame(Protocol):
     @property
-    def frame_data(self) -> FrameData: ...
+    def data(self) -> bytes: ...
     @property
     def metadata(self) -> FrameMetadata: ...
     @property
+    def info(self) -> ImageInformation: ...
+    @property
+    def timestamp(self) -> int: ...
+    @property
     def input_width(self) -> int: ...
     @property
     def input_height(self) -> int: ...
@@ -543,7 +552,7 @@ class VideoFrame(Protocol):
 
 class MotionFrame(VideoFrame, Protocol):
     @property
-    def motion(self) -> MotionState: ...
+    def motion(self) -> Optional[MotionSetEvent]: ...
 
 
 class CameraInputSettings(TypedDict):
@@ -560,16 +569,16 @@ class CameraConfigInputSettings(TypedDict):
     urls: list[str]
 
 
-class BaseCameraConfig(TypedDict, total=False):
+class BaseCameraConfig(TypedDict):
     name: str
-    nativeId: str
-    isCloud: bool
-    hasLight: bool
-    hasSiren: bool
-    hasBinarySensor: bool
-    hasBattery: bool
-    disabled: bool
-    info: CameraInformation
+    nativeId: Optional[str]
+    isCloud: Optional[bool]
+    hasLight: Optional[bool]
+    hasSiren: Optional[bool]
+    hasBinarySensor: Optional[bool]
+    hasBattery: Optional[bool]
+    disabled: Optional[bool]
+    info: Optional[CameraInformation]
 
 
 class CameraConfig(BaseCameraConfig):
@@ -652,6 +661,7 @@ class FfmpegOptions(TypedDict):
 class ReturnAudioFFmpegOptions(TypedDict):
     ffmpegPath: str
     input: list[SpawnInput]
+    logPrefix: Optional[str]
 
 
 class BaseCamera(TypedDict):
@@ -673,11 +683,11 @@ class BaseCamera(TypedDict):
 
 
 class Camera(BaseCamera):
-    hasAudioDetector: bool
-    hasMotionDetector: bool
-    hasObjectDetector: bool
-    extendedMotionDetector: bool
-    hasPtz: bool
+    cameraControllerPluginId: Optional[str]
+    audioDetectionPluginId: Optional[str]
+    motionDetectionPluginId: Optional[str]
+    objectDetectionPluginId: Optional[str]
+    ptzPluginId: Optional[str]
     sources: list[CameraInput]
     detectionZones: list[DetectionZone]
 
@@ -724,6 +734,8 @@ class CameraDevice(Protocol):
     @property
     def connected(self) -> bool: ...
     @property
+    def frameworker_connected(self) -> bool: ...
+    @property
     def disabled(self) -> bool: ...
     @property
     def name(self) -> str: ...
@@ -746,8 +758,6 @@ class CameraDevice(Protocol):
     @property
     def has_motion_detector(self) -> bool: ...
     @property
-    def extended_motion_detector(self) -> bool: ...
-    @property
     def has_audio_detector(self) -> bool: ...
     @property
     def has_object_detector(self) -> bool: ...
@@ -762,14 +772,14 @@ class CameraDevice(Protocol):
     @property
     def stream_source(self) -> CameraDeviceSource: ...
     @property
-    def snapshot_source(self) -> Optional[CameraInput]: ...
-    @property
     def high_resolution_source(self) -> Optional[CameraDeviceSource]: ...
     @property
     def mid_resolution_source(self) -> Optional[CameraDeviceSource]: ...
     @property
     def low_resolution_source(self) -> Optional[CameraDeviceSource]: ...
     @property
+    def snapshot_source(self) -> Optional[CameraInput]: ...
+    @property
     def ptz(self) -> CameraPTZDelegate: ...
     @property
     def sources(self) -> list[CameraDeviceSource]: ...
@@ -777,6 +787,7 @@ class CameraDevice(Protocol):
     def internal_sources(self) -> list[CameraDeviceInternalSource]: ...
 
     on_connected: HybridObservable[bool]
+    on_frameworker_connected: HybridObservable[bool]
     on_light_switched: HybridObservable[LightState]
     on_motion_detected: HybridObservable[MotionState]
     on_audio_detected: HybridObservable[AudioState]
@@ -857,8 +868,14 @@ class CameraDevice(Protocol):
     async def connect(self) -> None: ...
     async def disconnect(self) -> None: ...
 
-    def get_frames(self, prebuffer_duration: Optional[int] = None) -> AsyncGenerator[VideoFrame, None]: ...
-    def get_motion_frames(self) -> AsyncGenerator[MotionFrame, None]: ...
+    @overload
+    async def get_frames(
+        self, frame_type: Literal["video"], options: Optional[ImageOptions] = None
+    ) -> AsyncGenerator[VideoFrame, None]: ...
+    @overload
+    async def get_frames(
+        self, frame_type: Literal["motion"], options: Optional[ImageOptions] = None
+    ) -> AsyncGenerator[MotionFrame, None]: ...
 
     @overload
     async def update_state(
@@ -903,20 +920,6 @@ class CameraDevice(Protocol):
         state_name: Literal["battery"],
         event_data: BatterySetEvent,
     ) -> None: ...
-    async def update_state(
-        self,
-        state_name: Literal["light", "motion", "audio", "object", "doorbell", "siren", "battery"],
-        event_data: Union[
-            LightSetEvent,
-            MotionSetEvent,
-            AudioSetEvent,
-            ObjectSetEvent,
-            DoorbellSetEvent,
-            SirenSetEvent,
-            BatterySetEvent,
-        ],
-        frame: Optional[VideoFrame] = None,
-    ) -> None: ...
 
     async def add_camera_source(self, source: CameraConfigInputSettings) -> None: ...
     async def update_camera_source(
@@ -961,6 +964,16 @@ class DeviceManager(Protocol):
     def remove_all_listeners(self, event: Optional[DeviceManagerEventType] = None) -> None: ...
 
 
+@runtime_checkable
+class CoreManager(Protocol):
+    async def get_ffmpeg_path(self) -> str: ...
+    async def get_hwaccel_info(
+        self, target_codec: Union[Literal["h264"], Literal["h265"]]
+    ) -> "FfmpegArgs": ...
+    async def get_server_addresses(self) -> list[str]: ...
+    async def get_ice_servers(self) -> list["IceServer"]: ...
+
+
 class FfmpegArgs(TypedDict):
     codec: str
     hwaccel: str
@@ -976,14 +989,6 @@ class IceServer(TypedDict):
     credential: Optional[str]
 
 
-@runtime_checkable
-class CoreManager(Protocol):
-    async def get_ffmpeg_path(self) -> str: ...
-    async def get_hwaccel_info(self, target_codec: Union[Literal["h264"], Literal["h265"]]) -> FfmpegArgs: ...
-    async def get_server_addresses(self) -> list[str]: ...
-    async def get_ice_servers(self) -> list[IceServer]: ...
-
-
 # Schema related types and interfaces
 PluginConfig = dict[str, Any]
 
@@ -1227,7 +1232,6 @@ class CameraStorage(Protocol, Generic[V2]):
     values: V2
     schema: JsonSchemaForm
 
-    def initializeStorage(self) -> None: ...
     @overload
     async def getValue(self, path: str) -> Union[V1, None]: ...
     @overload
@@ -1264,4 +1268,4 @@ class StorageController(Protocol[S]):
         schema: Optional[JsonSchemaForm] = None,
     ) -> S: ...
     def get_camera_storage(self, camera_id: str) -> Optional[S]: ...
-    def remove_camera_storage(self, camera_id: str) -> None: ...
+    async def remove_camera_storage(self, camera_id: str) -> None: ...
